package cc.smtweb.framework.core.db.cache;

import cc.smtweb.framework.core.annotation.SwCache;
import cc.smtweb.framework.core.cache.AbstractCache;
import cc.smtweb.framework.core.cache.CacheManager;
import cc.smtweb.framework.core.db.DbEngine;
import cc.smtweb.framework.core.db.vo.ModelTable;
import org.springframework.dao.DataAccessException;
import org.springframework.jdbc.core.ResultSetExtractor;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;

/**
 * Created by Akmm at 2022/1/12 18:34
 */
@SwCache(ident = "ASP_MODEL_TABLE", title = "数据库表定义")
public class ModelTableCache extends AbstractCache<ModelTable> {
    private final static String mk = "k";
    private final static String md = "d";

    public static ModelTableCache getInstance() {
        return CacheManager.getIntance().getCache(ModelTableCache.class);
    }

    public ModelTableCache() {
        regMap(mk, k-> k.getName().toUpperCase());
        regList(md, k-> String.valueOf(k.getDatabaseId()));
    }

    @Override
    protected String getId(ModelTable bean) {
        return String.valueOf(bean.getId());
    }

    @Override
    protected List<ModelTable> loadAll() {
        return DbEngine.getInstance().query("SELECT\n" +
                "t.table_id,\n" +
                "t.prjoect_id,\n" +
                "t.catalog_id,\n" +
                "t.database_id,\n" +
                "t.table_extends,\n" +
                "t.table_name,\n" +
                "t.table_title,\n" +
                "t.table_abbr,\n" +
                "t.table_type,\n" +
                "t.need_cache,\n" +
                "t.table_content,\n" +
                "t.table_create_uid,\n" +
                "t.table_update_uid,\n" +
                "t.table_create_at,\n" +
                "t.table_update_at\n" +
                "from asp_model_table t\n", new ResultSetExtractor<List<ModelTable>>() {
            @Override
            public List<ModelTable> extractData(ResultSet rs) throws SQLException, DataAccessException {
                List<ModelTable> list = new ArrayList<>();
                while (rs.next()) {
                    ModelTable table = new ModelTable();
                    list.add(table);
                    table.setId(rs.getLong("table_id"));
                    table.setDatabaseId(rs.getLong("database_id"));
                    table.setPrjoectId(rs.getLong("prjoect_id"));
                    table.setCatalogId(rs.getLong("catalog_id"));
                    table.setTableExtends(rs.getString("table_extends"));
                    table.setName(rs.getString("table_name").toUpperCase());
                    table.setTitle(rs.getString("table_title"));
                    table.setAbbr(rs.getString("table_abbr"));
                    table.setType(rs.getInt("table_type"));
                    table.setNeedCache(rs.getInt("need_cache") == 1);
                    table.setCreateUid(rs.getLong("table_create_uid"));
                    table.setCreateAt(rs.getLong("table_create_at"));
                    table.setUpdateAt(rs.getLong("table_update_at"));
                    table.setTableContent(rs.getString("table_content"));
                }
                return list;
            }
        });
    }

    public final ModelTable getByName(String key) {
        return localGetByKey(mk, key.toUpperCase());
    }
    public final Set<ModelTable> getDbTables(long dbId) {
        return localGetListByKey(md, String.valueOf(dbId));
    }
}
